import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import scipy.optimize as so

X = 1
N = 100

d_func = ss.beta(X+1, N-X+1)

t_range = np.linspace(0, 0.05)
t_v = d_func.pdf(t_range)

mean_v = (X+1) / (N+2)
mode_v = X / N
print('mean = ', mean_v)
print('mode = ', mode_v)


class Func_med:
    def __init__(self, f):
        self.f = f
    def v(self, t):
        return self.f(t) - 0.5

N = 100

p_true_range = [v/100 for v in range(1, 100)]

print(p_true_range)

bias_mean = []
bias_median = []
bias_mode = []
RMSE_mean = []
RMSE_median = []
RMSE_mode = []
RMSER_mean = []
RMSER_median = []
RMSER_mode = []

for p_true in p_true_range:
    print(p_true, end = '\r')
    v_sum_mean = 0.0
    v_sum_mode = 0.0
    v_sum_med = 0.0
    ssum_mean = 0.0
    ssum_mode = 0.0
    ssum_med = 0.0
    for X in range(0, N + 1):
        p = ss.binom.pmf(X, N, p_true)
        #print('p =', p)
        v_sum_mean += p * (X + 1) / (N + 2)
        v_sum_mode += p * X / N
        ssum_mean += p * ((X +1) / (N+2) - p_true) ** 2
        ssum_mode += p * ((X / N) - p_true) ** 2

        
        d_func = ss.beta(X+1, N-X+1)
        func_med = Func_med(d_func.cdf)
        f_med = func_med.v
        v_med = so.bisect(f_med, 0.0, 1.0)
        v_sum_med += p * v_med
        ssum_med += p * (v_med - p_true) ** 2
        
    bias_mean.append(v_sum_mean - p_true)
    bias_median.append(v_sum_med - p_true)
    bias_mode.append(v_sum_mode - p_true)
    RMSE_mean.append(ssum_mean ** 0.5)
    RMSE_median.append(ssum_med ** 0.5)
    RMSE_mode.append(ssum_mode ** 0.5)
    RMSER_mean.append((ssum_mean ** 0.5) / p_true)
    RMSER_median.append((ssum_med ** 0.5) / p_true)
    RMSER_mode.append((ssum_mode ** 0.5) / p_true)


N2 = 200

p_true_range = [v/100 for v in range(1, 100)]

print(p_true_range)

bias_mean2 = []
bias_median2 = []
bias_mode2 = []
RMSE_mean2 = []
RMSE_median2 = []
RMSE_mode2 = []
RMSER_mean2 = []
RMSER_median2 = []
RMSER_mode2 = []

for p_true in p_true_range:
    print(f'{p_true} for N2', end = '\r')
    v_sum_mean = 0.0
    v_sum_mode = 0.0
    v_sum_med = 0.0
    ssum_mean = 0.0
    ssum_mode = 0.0
    ssum_med = 0.0
    for X in range(0, N2 + 1):
        p = ss.binom.pmf(X, N2, p_true)
        v_sum_mean += p * (X + 1) / (N2 + 2)
        v_sum_mode += p * X / N2
        ssum_mean += p * ((X +1) / (N2+2) - p_true) ** 2
        ssum_mode += p * ((X / N2) - p_true) ** 2

        
        d_func = ss.beta(X+1, N2-X+1)
        func_med = Func_med(d_func.cdf)
        f_med = func_med.v
        v_med = so.bisect(f_med, 0.0, 1.0)
        v_sum_med += p * v_med
        ssum_med += p * (v_med - p_true) ** 2
        
    bias_mean2.append(v_sum_mean - p_true)
    bias_median2.append(v_sum_med - p_true)
    bias_mode2.append(v_sum_mode - p_true)
    RMSE_mean2.append(ssum_mean ** 0.5)
    RMSE_median2.append(ssum_med ** 0.5)
    RMSE_mode2.append(ssum_mode ** 0.5)
    RMSER_mean2.append((ssum_mean ** 0.5) / p_true)
    RMSER_median2.append((ssum_med ** 0.5) / p_true)
    RMSER_mode2.append((ssum_mode ** 0.5) / p_true)

plt.title('Bias', fontsize = 16)
plt.plot(p_true_range, bias_mean, c = 'k', ls = (0,(4,1)), lw = 2, label = 'mean(N=100)')
plt.plot(p_true_range, bias_median, c = 'k', ls = (0,(4,1,2,1)), lw = 2, label = 'median(N=100)')
plt.plot(p_true_range, bias_mean2, c = 'k', ls = (0,(4,1)), lw = 1, label = 'mean(N=200)')
plt.plot(p_true_range, bias_median2, c = 'k', ls = (0,(4,1,2,1)), lw = 1, label = 'median(N=200)')
plt.plot(p_true_range, bias_mode, c = 'k', ls = (0,(1,2)), lw = 2, label = 'MAP(N=100)')
plt.plot(p_true_range, bias_mode2, c = 'k', ls = (2,(2,2)), lw = 1, label = 'MAP(N=200)')
plt.xticks(fontsize = 14)
plt.xlabel(r'$\theta_0$', fontsize = 16)
plt.legend()
plt.tight_layout()
plt.savefig('Figure_2.png')
plt.show()

plt.title('RMS Error', fontsize = 16)

plt.plot(p_true_range, RMSE_mode, c = 'k', ls = (0,(1,2)), lw = 2, label = 'MAP(N=100)')
plt.plot(p_true_range, RMSE_median, c = 'k', ls = (0,(4,1,2,1)), lw = 2, label = 'median(N=100)')
plt.plot(p_true_range, RMSE_mean, c = 'k', ls = (0,(4,1)), lw = 2, label = 'mean(N=100)')
plt.plot(p_true_range, RMSE_mode2, c = 'k', ls = (2,(2,2)), lw = 1, label = 'MAP(N=200)')
plt.plot(p_true_range, RMSE_median2, c = 'k', ls = (0,(4,1,2,1)), lw = 1, label = 'median(N=200)')
plt.plot(p_true_range, RMSE_mean2, c = 'k', ls = (0,(4,1)), lw = 1, label = 'mean(N=200)')
plt.legend(fontsize = 12)
plt.xlabel(r'$θ_0$', fontsize = 13)
plt.tight_layout()
plt.savefig('Figure_3.png')
plt.show()

plt.title(r'RMS Error / $θ_0$', fontsize = 16)

plt.plot(p_true_range[:20], RMSER_mean[:20], c = 'k', ls = (0,(4,1)), lw = 2, label = 'mean(N=100)') 
plt.plot(p_true_range[:20], RMSER_median[:20], c = 'k', ls = (0,(4,1,2,1)), lw = 2, label = 'median(N=100)')
plt.plot(p_true_range[:20], RMSER_mode[:20], c = 'k', ls = (0,(1,2)), lw = 2, label = 'MAP(N=100)')
plt.plot(p_true_range[:20], RMSER_mean2[:20], c = 'k', ls = (0,(4,1)), lw = 1, label = 'mean(N=200)')
plt.plot(p_true_range[:20], RMSER_median2[:20], c = 'k', ls = (0,(4,1,2,1)), lw = 1, label = 'median(N=200)')
plt.plot(p_true_range[:20], RMSER_mode2[:20], c = 'k', ls = (2,(2,2)), lw = 1, label = 'MAP(N=200)')
plt.xlabel(r'$\theta_0$', fontsize = 14)
plt.legend(fontsize = 14)
plt.tight_layout()
plt.savefig('Figure4.png')
plt.show()
